@@ -8,6 +8,7 @@ A history-aware add command to help with data entry.
 module Hledger.Cli.Add
 where
 
+import Control.Applicative ((<*))
 import Control.Exception as E
 import Control.Monad
 import Control.Monad.Trans (liftIO)
@@ -178,7 +179,7 @@ dateAndCodeWizard EntryState{..} = do
     where
       parseSmartDateAndCode refdate s = either (const Nothing) (\(d,c) -> return (fixSmartDate refdate d, c)) edc
           where
-            edc = runParser dateandcodep nullctx "" $ lowercase s
+            edc = runParser (dateandcodep <* eof) nullctx "" $ lowercase s
             dateandcodep :: Stream [Char] m t => ParsecT [Char] JournalContext m (SmartDate, String)
             dateandcodep = do
                 d <- smartdate
@@ -242,7 +243,7 @@ accountWizard EntryState{..} = do
       parseAccountOrDotOrNull _  _ "."       = dbg $ Just "." -- . always signals end of txn
       parseAccountOrDotOrNull "" True ""     = dbg $ Just ""  -- when there's no default and txn is balanced, "" also signals end of txn
       parseAccountOrDotOrNull def@(_:_) _ "" = dbg $ Just def -- when there's a default, "" means use that
-      parseAccountOrDotOrNull _ _ s          = dbg $ either (const Nothing) validateAccount $ runParser accountnamep (jContext esJournal) "" s -- otherwise, try to parse the input as an accountname
+      parseAccountOrDotOrNull _ _ s          = dbg $ either (const Nothing) validateAccount $ runParser (accountnamep <* eof) (jContext esJournal) "" s -- otherwise, try to parse the input as an accountname
       dbg = id -- strace
       validateAccount s | no_new_accounts_ esOpts && not (s `elem` journalAccountNames esJournal) = Nothing
                         | otherwise = Just s
@@ -266,7 +267,7 @@ amountAndCommentWizard EntryState{..} = do
    maybeRestartTransaction $
    line $ green $ printf "Amount  %d%s: " pnum (showDefault def)
     where
-      parseAmountAndComment = either (const Nothing) Just . runParser amountandcommentp nodefcommodityctx ""
+      parseAmountAndComment = either (const Nothing) Just . runParser (amountandcommentp <* eof) nodefcommodityctx ""
       nodefcommodityctx = (jContext esJournal){ctxDefaultCommodityAndStyle=Nothing}
       amountandcommentp :: Stream [Char] m t => ParsecT [Char] JournalContext m (Amount, String)
       amountandcommentp = do
