@@ -129,7 +129,7 @@ There are three kinds of multi-column balance report, indicated by the heading:
   period. Here, checking's balance increased by 10 in Feb:
 
   > Change of balance (flow):
-  > 
+  >
   >                  Jan   Feb   Mar
   > assets:checking   20    10    -5
 
@@ -138,7 +138,7 @@ There are three kinds of multi-column balance report, indicated by the heading:
   Here, 30 is the sum of checking postings during Jan and Feb:
 
   > Ending balance (cumulative):
-  > 
+  >
   >                  Jan   Feb   Mar
   > assets:checking   20    30    25
 
@@ -148,7 +148,7 @@ There are three kinds of multi-column balance report, indicated by the heading:
   pre-Jan postings which created a starting balance of 100:
 
   > Ending balance (historical):
-  > 
+  >
   >                  Jan   Feb   Mar
   > assets:checking  120   130   125
 
@@ -161,7 +161,7 @@ Here's a (imperfect?) specification for the eliding/omitting behaviour:
 * An account less deep than the report's max depth, with just one
 interesting subaccount, and the same balance as the subaccount, is
 non-interesting, and prefixed to the subaccount's line, unless
-@--no-elide@ is in effect. 
+@--no-elide@ is in effect.
 
 * An account with a zero inclusive balance and less than two interesting
 subaccounts is not displayed at all, unless @--empty@ is in effect.
@@ -242,20 +242,17 @@ module Hledger.Cli.Balance (
  ,tests_Hledger_Cli_Balance
 ) where
 
-import Data.List
-import Data.Maybe
--- import System.Console.CmdArgs
 import System.Console.CmdArgs.Explicit as C
--- import System.Console.CmdArgs.Text
+import Text.CSV
 import Test.HUnit
+import Text.Printf (printf)
 import Text.Tabular as T
 import Text.Tabular.AsciiArt
 
 import Hledger
-import Prelude hiding (putStr)
-import Hledger.Utils.UTF8IOCompat (putStr)
 import Hledger.Data.OutputFormat
 import Hledger.Cli.Options
+import Hledger.Cli.Utils
 
 
 -- | Command line options for this command.
@@ -272,6 +269,7 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
      ,flagNone ["cumulative"] (\opts -> setboolopt "cumulative" opts) "multicolumn mode: show accumulated ending balances"
      ,flagNone ["historical","H"] (\opts -> setboolopt "historical" opts) "multicolumn mode: show historical ending balances"
      ]
+     ++ outputflags
     ,groupHidden = []
     ,groupNamed = [generalflagsgroup1]
     }
@@ -280,24 +278,48 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
 
 -- | The balance command, prints a balance report.
 balance :: CliOpts -> Journal -> IO ()
-balance CliOpts{reportopts_=ropts} j = do
+balance opts@CliOpts{reportopts_=ropts} j = do
   d <- getCurrentDay
-  let output =
-       case formatFromOpts ropts of
-         Left err -> [err]
-         Right _ ->
-          case (intervalFromOpts ropts, balancetype_ ropts) of
-            (NoInterval,_)        -> balanceReportAsText           ropts  $ balanceReport ropts (queryFromOpts d ropts) j
-            (_,PeriodBalance)     -> periodBalanceReportAsText     ropts $ multiBalanceReport ropts (queryFromOpts d ropts) j
-            (_,CumulativeBalance) -> cumulativeBalanceReportAsText ropts $ multiBalanceReport ropts (queryFromOpts d ropts) j
-            (_,HistoricalBalance) -> historicalBalanceReportAsText ropts $ multiBalanceReport ropts (queryFromOpts d ropts) j
-  putStr $ unlines output
+  case lineFormatFromOpts ropts of
+    Left err -> error' $ unlines [err]
+    Right _ -> do
+      let format   = outputFormatFromOpts opts
+          interval = intervalFromOpts ropts
+          baltype  = balancetype_ ropts
+      case interval of
+        NoInterval -> do
+          let report = balanceReport ropts (queryFromOpts d ropts) j
+              render = case format of
+                "csv" -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
+                _     -> balanceReportAsText
+          writeOutput opts $ render ropts report
+        _ -> do
+          let report = multiBalanceReport ropts (queryFromOpts d ropts) j
+              render = case format of
+                "csv" -> \ropts r -> (++ "\n") $ printCSV $ multiBalanceReportAsCsv ropts r
+                _     -> case baltype of
+                  PeriodBalance     -> periodBalanceReportAsText
+                  CumulativeBalance -> cumulativeBalanceReportAsText
+                  HistoricalBalance -> historicalBalanceReportAsText
+          writeOutput opts $ render ropts report
 
--- | Render an old-style single-column balance report as plain text.
-balanceReportAsText :: ReportOpts -> BalanceReport -> [String]
-balanceReportAsText opts ((items, total)) = concat lines ++ t
+-- single-column balance reports
+
+-- | Render a single-column balance report as CSV.
+balanceReportAsCsv :: ReportOpts -> BalanceReport -> CSV
+balanceReportAsCsv opts (items, total) =
+  ["account","balance"] :
+  [[a, showMixedAmountWithoutPrice b] | ((a, _, _), b) <- items]
+  ++
+  if no_total_ opts
+  then []
+  else [["total", showMixedAmountOneLineWithoutPrice total]]
+
+-- | Render a single-column balance report as plain text.
+balanceReportAsText :: ReportOpts -> BalanceReport -> String
+balanceReportAsText opts ((items, total)) = unlines $ concat lines ++ t
   where
-      lines = case formatFromOpts opts of
+      lines = case lineFormatFromOpts opts of
                 Right f -> map (balanceReportItemAsText opts f) items
                 Left err -> [[err]]
       t = if no_total_ opts
@@ -314,6 +336,7 @@ tests_balanceReportAsText = [
       "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
     let opts = defreportopts
     balanceReportAsText opts (balanceReport opts (queryFromOpts (parsedate "2008/11/26") opts) j) `is`
+      unlines
       ["                -100  актив:наличные"
       ,"                 100  расходы:покупки"
       ,"--------------------"
@@ -337,7 +360,7 @@ balanceReportItemAsText opts format ((_, accountName, depth), Mixed amounts) =
     -- 'amounts' could contain several quantities of the same commodity with different price.
     -- In order to combine them into single value (which is expected) we take the first price and
     -- use it for the whole mixed amount. This could be suboptimal. XXX
-    let Mixed normAmounts = normaliseMixedAmountPreservingFirstPrice (Mixed amounts) in
+    let Mixed normAmounts = normaliseMixedAmountSquashPricesForDisplay (Mixed amounts) in
     case normAmounts of
       [] -> []
       [a] -> [formatBalanceReportItem opts (Just accountName) depth a format]
@@ -359,22 +382,35 @@ formatBalanceReportItem opts accountName depth amount (fmt:fmts) =
 
 formatField :: ReportOpts -> Maybe AccountName -> Int -> Amount -> Bool -> Maybe Int -> Maybe Int -> HledgerFormatField -> String
 formatField opts accountName depth total ljust min max field = case field of
-        AccountField     -> formatValue ljust min max $ maybe "" (accountNameDrop (drop_ opts)) accountName
+        AccountField     -> formatValue ljust min max $ maybe "" (maybeAccountNameDrop opts) accountName
         DepthSpacerField -> case min of
                                Just m  -> formatValue ljust Nothing max $ replicate (depth * m) ' '
                                Nothing -> formatValue ljust Nothing max $ replicate depth ' '
         TotalField       -> formatValue ljust min max $ showAmountWithoutPrice total
         _                  -> ""
 
+-- multi-column balance reports
+
+-- | Render a multi-column balance report as CSV.
+multiBalanceReportAsCsv :: ReportOpts -> MultiBalanceReport -> CSV
+multiBalanceReportAsCsv opts (MultiBalanceReport (colspans, items, coltotals)) =
+  ("account" : "short account" : "indent" : map showDateSpan colspans) :
+  [a : a' : show i : map showMixedAmountOneLineWithoutPrice amts | ((a,a',i), amts) <- items]
+  ++
+  if no_total_ opts
+  then []
+  else [["totals", "", ""] ++ map showMixedAmountOneLineWithoutPrice coltotals]
+
 -- | Render a multi-column period balance report as plain text suitable for console output.
-periodBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> [String]
-periodBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltotals)) =
-  (["Change of balance (flow):"] ++) $
+periodBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
+periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, coltotals)) =
+  unlines $
+  ([printf "Balance changes in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
   trimborder $ lines $
    render
     id
     ((" "++) . showDateSpan)
-    showMixedAmountWithoutPrice
+    showMixedAmountOneLineWithoutPrice
     $ Table
       (T.Group NoLine $ map (Header . padright acctswidth) accts)
       (T.Group NoLine $ map Header colspans)
@@ -388,18 +424,19 @@ periodBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltotals))
     accts = map renderacct items'
     renderacct ((a,a',i),_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
-      | otherwise  = accountNameDrop (drop_ opts) a
+      | otherwise  = maybeAccountNameDrop opts a
     acctswidth = maximum $ map length $ accts
     totalrow | no_total_ opts = row "" []
              | otherwise      = row "" coltotals
 
 -- | Render a multi-column cumulative balance report as plain text suitable for console output.
-cumulativeBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> [String]
-cumulativeBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltotals)) =
-  (["Ending balance (cumulative):"] ++) $
+cumulativeBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
+cumulativeBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, coltotals)) =
+  unlines $
+  ([printf "Ending balances (cumulative) in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
   trimborder $ lines $
-   render id ((" "++) . maybe "" (showDate . prevday) . spanEnd) showMixedAmountWithoutPrice $
-    addtotalrow $ 
+   render id ((" "++) . maybe "" (showDate . prevday) . spanEnd) showMixedAmountOneLineWithoutPrice $
+    addtotalrow $
      Table
        (T.Group NoLine $ map (Header . padright acctswidth) accts)
        (T.Group NoLine $ map Header colspans)
@@ -409,18 +446,19 @@ cumulativeBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltota
     accts = map renderacct items
     renderacct ((a,a',i),_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
-      | otherwise  = accountNameDrop (drop_ opts) a
+      | otherwise  = maybeAccountNameDrop opts a
     acctswidth = maximum $ map length $ accts
     addtotalrow | no_total_ opts = id
                 | otherwise      = (+----+ row "" coltotals)
 
 -- | Render a multi-column historical balance report as plain text suitable for console output.
-historicalBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> [String]
-historicalBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltotals)) =
-  (["Ending balance (historical):"] ++) $
+historicalBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
+historicalBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, coltotals)) =
+  unlines $
+  ([printf "Ending balances (historical) in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
   trimborder $ lines $
-   render id ((" "++) . maybe "" (showDate . prevday) . spanEnd) showMixedAmountWithoutPrice $
-    addtotalrow $ 
+   render id ((" "++) . maybe "" (showDate . prevday) . spanEnd) showMixedAmountOneLineWithoutPrice $
+    addtotalrow $
      Table
        (T.Group NoLine $ map (Header . padright acctswidth) accts)
        (T.Group NoLine $ map Header colspans)
@@ -430,11 +468,16 @@ historicalBalanceReportAsText opts (MultiBalanceReport (colspans, items, coltota
     accts = map renderacct items
     renderacct ((a,a',i),_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
-      | otherwise  = accountNameDrop (drop_ opts) a
+      | otherwise  = maybeAccountNameDrop opts a
     acctswidth = maximum $ map length $ accts
     addtotalrow | no_total_ opts = id
                 | otherwise      = (+----+ row "" coltotals)
 
+-- | Figure out the overall date span of a multicolumn balance report.
+multiBalanceReportSpan :: MultiBalanceReport -> DateSpan
+multiBalanceReportSpan (MultiBalanceReport ([], _, _))       = DateSpan Nothing Nothing
+multiBalanceReportSpan (MultiBalanceReport (colspans, _, _)) = DateSpan (spanStart $ head colspans) (spanEnd $ last colspans)
+
 
 tests_Hledger_Cli_Balance = TestList
   tests_balanceReportAsText
