@@ -1,4 +1,4 @@
-{-| 
+{-|
 
 A ledger-compatible @print@ command.
 
@@ -14,17 +14,17 @@ where
 import Data.List
 import System.Console.CmdArgs.Explicit
 import Test.HUnit
+import Text.CSV
 
 import Hledger
-import Prelude hiding (putStr)
-import Hledger.Utils.UTF8IOCompat (putStr)
 import Hledger.Cli.Options
+import Hledger.Cli.Utils
 
 
 printmode = (defCommandMode $ ["print"] ++ aliases) {
   modeHelp = "show transaction entries" `withAliases` aliases
  ,modeGroupFlags = Group {
-     groupUnnamed = []
+     groupUnnamed = outputflags
     ,groupHidden = []
     ,groupNamed = [generalflagsgroup1]
     }
@@ -33,20 +33,24 @@ printmode = (defCommandMode $ ["print"] ++ aliases) {
 
 -- | Print journal transactions in standard format.
 print' :: CliOpts -> Journal -> IO ()
-print' CliOpts{reportopts_=ropts} j = do
+print' opts@CliOpts{reportopts_=ropts} j = do
   d <- getCurrentDay
   let q = queryFromOpts d ropts
-  putStr $ entriesReportAsText ropts q $ entriesReport ropts q j
+      fmt = outputFormatFromOpts opts
+      (render, ropts') = case fmt of
+        "csv" -> ((++"\n") . printCSV . entriesReportAsCsv, ropts{accountlistmode_=ALFlat})
+        _     -> (entriesReportAsText,                      ropts)
+  writeOutput opts $ render $ entriesReport ropts' q j
 
-entriesReportAsText :: ReportOpts -> Query -> EntriesReport -> String
-entriesReportAsText _ _ items = concatMap showTransactionUnelided items
+entriesReportAsText :: EntriesReport -> String
+entriesReportAsText items = concatMap showTransactionUnelided items
 
 -- XXX
 -- tests_showTransactions = [
 --   "showTransactions" ~: do
 
 --    -- "print expenses" ~:
---    do 
+--    do
 --     let opts = defreportopts{query_="expenses"}
 --     d <- getCurrentDay
 --     showTransactions opts (queryFromOpts d opts) samplejournal `is` unlines
@@ -58,7 +62,7 @@ entriesReportAsText _ _ items = concatMap showTransactionUnelided items
 --      ]
 
 --   -- , "print report with depth arg" ~:
---    do 
+--    do
 --     let opts = defreportopts{depth_=Just 2}
 --     d <- getCurrentDay
 --     showTransactions opts (queryFromOpts d opts) samplejournal `is` unlines
@@ -82,5 +86,39 @@ entriesReportAsText _ _ items = concatMap showTransactionUnelided items
 --       ]
 --  ]
 
+entriesReportAsCsv :: EntriesReport -> CSV
+entriesReportAsCsv items =
+  concat $
+  ([["nth","date","date2","status","code","description","comment","account","amount","commodity","credit","debit","status","posting-comment"]]:).snd $
+  mapAccumL (\n e -> (n + 1, transactionToCSV n e)) 0 items
+
+transactionToCSV :: Integer -> Transaction -> CSV
+transactionToCSV n t =
+	map (\p -> show n:date:date2:status:code:description:comment:p)
+	 (concatMap postingToCSV $ tpostings t)
+	where
+		description = tdescription t
+		date = showDate (tdate t)
+		date2 = maybe "" showDate (tdate2 t)
+		status = if tstatus t then "*" else ""
+		code = tcode t
+		comment = chomp $ strip $ tcomment t
+
+postingToCSV :: Posting -> CSV
+postingToCSV p =
+	map (\(a@(Amount {aquantity=q,acommodity=c})) ->
+		let a_ = a{acommodity=""} in
+		let amount = showAmount a_ in
+		let commodity = c in
+		let credit = if q < 0 then showAmount $ negate a_ else "" in
+		let debit  = if q > 0 then showAmount a_ else "" in
+		account:amount:commodity:credit:debit:status:comment:[])
+	 amounts
+	where
+		Mixed amounts = pamount p
+		status = if pstatus p then "*" else ""
+		account = showAccountName Nothing (ptype p) (paccount p)
+		comment = chomp $ strip $ pcomment p
+
 tests_Hledger_Cli_Print = TestList []
   -- tests_showTransactions
