@@ -9,6 +9,7 @@ Hledger.Utils.
 module Hledger.Cli.Utils
     (
      withJournalDo,
+     writeOutput,
      journalReload,
      journalReloadIfChanged,
      journalFileIsNewer,
@@ -34,6 +35,8 @@ import System.Process (readProcessWithExitCode)
 import System.Time (ClockTime, getClockTime, diffClockTimes, TimeDiff(TimeDiff))
 import Test.HUnit
 import Text.Printf
+import Text.Regex.TDFA ((=~))
+
 
 -- kludge - adapt to whichever directory version is installed, or when
 -- cabal macros aren't available, assume the new directory
@@ -65,16 +68,22 @@ withJournalDo opts cmd = do
   -- to let the add command work.
   rulespath <- rulesFilePathFromOpts opts
   journalpath <- journalFilePathFromOpts opts
-  ej <- readJournalFile Nothing rulespath journalpath
+  ej <- readJournalFile Nothing rulespath (not $ ignore_assertions_ opts) journalpath
   either error' (cmd opts . journalApplyAliases (aliasesFromOpts opts)) ej
 
+-- | Write some output to stdout or to a file selected by --output-file.
+writeOutput :: CliOpts -> String -> IO ()
+writeOutput opts s = do
+  f <- outputFileFromOpts opts
+  (if f == "-" then putStr else writeFile f) s
+  
 -- -- | Get a journal from the given string and options, or throw an error.
 -- readJournalWithOpts :: CliOpts -> String -> IO Journal
 -- readJournalWithOpts opts s = readJournal Nothing Nothing Nothing s >>= either error' return
 
 -- | Re-read a journal from its data file, or return an error string.
 journalReload :: Journal -> IO (Either String Journal)
-journalReload j = readJournalFile Nothing Nothing $ journalFilePath j
+journalReload j = readJournalFile Nothing Nothing True $ journalFilePath j
 
 -- | Re-read a journal from its data file mostly, only if the file has
 -- changed since last read (or if there is no file, ie data read from
@@ -179,7 +188,8 @@ safeGetDirectoryContents "" = getDirectoryContents "."
 safeGetDirectoryContents fp = getDirectoryContents fp
 
 -- | Does the second file represent a backup of the first, and if so which version is it ?
+-- XXX nasty regex types intruding, add a simpler api to Hledger.Utils.Regex
 backupNumber :: FilePath -> FilePath -> Maybe Int
-backupNumber f g = case regexMatch ("^" ++ f ++ "\\.([0-9]+)$") g of
-                        Just (_, ((_,suffix):_)) -> readMay suffix
+backupNumber f g = case g =~ ("^" ++ f ++ "\\.([0-9]+)$") of
+                        (_::FilePath, _::FilePath, _::FilePath, [ext::FilePath]) -> readMay ext
                         _ -> Nothing
